function[fdatype]=fda1(pcatype, group, among, maxvar)
%fda1				- stepwise factorial discriminant analysis on PCA scores 
% function[fdatype]=fda1(pcatype, group, among, maxscore)
%
%Input arguments:
%---------------
%pcatype: (structure) output argument of function "pca" applied on the predictive data
%set
%group : SAISIR vector of group (integer values). Identical numbers mean
%that the observations belong to the same group. 
%among : (integer) Maximal rank (dimension) of PC-score allowed to enter in the model
%maxscore: (integer) Maximum number of scores allowed to enter in the model.
%
%Output arguments:
%----------------
%fdatype with fields:
%introduced: rank of the PC scores introduced in the model
%ncorrect: number of correct classifications (no validation) at each step
%beta: projection coefficients such as datafactor=X*beta 
%datafactor: discriminant scores
%centroidfactor: scores of the barycenters (centroids)
%eigenval: eigenvalues of the discriminant analysis
%confusion: confusion matrix (row actual; column predicted)
%average: average of the predicitive data set.  
%
% Assesses a stepwise factorial discriminant analysis according 
% to Bertrand et al., J of Chemometrics, Vol . 4, 413-427 (1990). 
% the basic idea is to assess a factorial discriminant analysis on the scores of 
% a previous pca. The criterion of score selection is the maximisation of 
% the trace of T-1B.  
% In order to avoid using PC-scores with very small eigenvalues, 
% the input argument "among" gives the maximal dimension to be allowed. 
% "maxscore" indicates the maximal number of scores. 
% "datafactor" corresponds to the final model 
%(with maxscore scores introduced). If one is interested in a more
% economical model, it is easy, looking at the classification, to reduce the
% value in "maxscore" and re-run "fda1". 
%
%Typical example:
%g=create_group1(wheat,1,3);%% creation of a grouping from the identifiers
%%names, using characters in position 1 
%p=pca(wheat); %% first PCA
%res=fda1(p,g,20,5);%% model with 5 scores introduced among the 20 first
%%ones
%res.ncorrect.d
% % ans =
% %          13.00  1 PC score introduced
% %          31.00
% %          69.00
% %          77.00
% %          93.00 5 PC scores introduced
%colored_map1(res.datafactor,1,2,1,3)% map of the discrimination
%figure;
%ellipse_map(res.datafactor,1,2,g,1,0.05) % shown as confidence ellipses
%
%Note : the number of dimensions in datafactor is less than the number of
%qualitative groups minus 1 
%(with 2 groups, only 1 discriminant dimension!).
%
%SEE ALSO maha3, maha6, plsda, quaddis, applyfda1. 

[nrow ncol]=size(pcatype.score.d);
C=pcatype.score.d(:,1:among);
Y=normc(C);
%bid=Y'*Y
ngroup=max(group.d);
aux=zeros(nrow,ngroup);
for i=1:nrow
   aux(i,group.d(i))=1;
end   
w=sum(aux);
P=diag(1./w)*aux'*Y;% gravity center of the normalised scores
diago=diag(P'*diag(w)*P);
[diago,rank]=sort(-diago);
aux=[];
%size(rank)
step=0;
predict=zeros(1,nrow);
for i=rank(1:maxvar)'
   disp([step+1 i maxvar]);
   aux=[aux i];
   P1=P(:,aux);   
   %P1=diag(sqrt(w))*P(:,aux);  vrifier ********************
   Y1=Y(:,aux);
   xpcatype=xpca(P1);% PCA on the gravity centers   
   S=Y1*xpcatype.eigenvec;% scores of the data 
   J=P1*xpcatype.eigenvec;% scores of the gravity centers
   %size(J)
   %size(S)
   for j=1:nrow
   	%if(mod(j,10000)==0)
    %    disp([j nrow]);
    %end
       delta=J-ones(ngroup,1)*S(j,:);   
      dist=sum((delta.*delta)',1);   
      [aux1 predicted_group]=min(dist);
   	predict(j)=predicted_group;   
   end      
step=step+1;   
ncorrect.d(step)=sum(predict'==group.d);
end
ranked=rank(1:maxvar);
V=pcatype.eigenvec.d(:,ranked)*diag(1./sqrt(pcatype.eigenval.d(ranked)))*xpcatype.eigenvec;
fdatype.introduced.d=rank(1:maxvar);
fdatype.introduced.i=pcatype.score.v(rank(1:maxvar),:);
fdatype.introduced.v='introduced component';
fdatype.ncorrect.i=pcatype.score.v(rank(1:maxvar),:);
fdatype.ncorrect.v='number of correct classifications';
fdatype.ncorrect.d=ncorrect.d';
fdatype.beta.d = V/sqrt(nrow);
fdatype.beta.i=pcatype.eigenvec.i; 
for i=1:size(V,2)
   chaine=['F' num2str(i) '        '];
   fdatype.beta.v(i,:)=chaine(1:6);
end
fdatype.datafactor.d=S;
fdatype.centroidfactor.d=J;
fdatype.centroidfactor.v=fdatype.beta.v;
fdatype.datafactor.i=pcatype.score.i;
fdatype.datafactor.v=fdatype.beta.v;
fdatype.eigenval.d=xpcatype.eigenval;
fdatype.eigenval.i='eigenvalues';
for i=1:ngroup
   chaine=['G' num2str(i) '        '];
   fdatype.centroidfactor.i(i,:)=chaine(1:6);
end

confusion=zeros(ngroup,ngroup);
for i=1:nrow
   confusion(group.d(i),predict(i))=confusion(group.d(i),predict(i))+1;
end
fdatype.confusion=matrix2saisir(confusion,'actual','predi ');
fdatype.average=pcatype.average;