function[ridgetype]=pca_ridge_regression(pcatype,y,range)
%pca_ridge_regression						- Basic ridge regression after PCA
%function[ridgetype]=pca_ridge_regression(pcatype,y,range)
%
%Input arguments
%===============
%pcatype: structure, output argument of function "PCA"
%y: variable to be predicted (n x 1)
%range: MATLAB vector of positive integers (1 x k)
%
%Output arguments:
%================
%ridgetype with fields:
%beta: coefficients of the model (p x k), applicable on X 
%krange: (1 X q) values of the coefficient k of ridge regressions
%averagex: average of original predictive variables (1 x p)
%averagey: average of y (1 x 1) 
%rmsec: root mean square of validation (1 x q)
%predy: y predicted (n x q)
%corr: correlation coefficient (1 x q] between predicted and observed
%values
%
% The function calculates as many ridge regression models as the number of
% elements in "range"
% In ridge regression, the product X'X is replaced by X'X +kI.
% In the present function, k is in fact the eigenvalue of the corresponding PCA component
% for example, if krange = [1 3 5], that means that the tested values of k
% are the eigenvalues #1, #3, #5.
% The rationale of this, is that k in ridge_regression is very difficult to find
% it is a good idea to test a value in the range of the observed eigenvalues
%
%%Typical example
%===============
%%Let DATA (n x p) be the SAISIR matrix of predictive variables and y (n x 1) the variable to be predicted.
%mypca=pca(DATA); %
%myrange=1:20 %% testing the eigenvalues from 1 to 20
%res=pca_ridge_regression(mypca,y,myrange)
%xdisp(res.rmsec.d);%% displaying (for example) the errors 
%
%See also ridge_regression, ridge_regression1, apply_ridge_regression,
%pca_cross_ridge_regression.
%
meanX=pcatype.average.d;
meanY=mean(y.d);
X=pcatype.score.d;%-ones(n,1)*meanX;
[n,p]=size(X);
Y=y.d;%-ones(n,1)*meanY;
tmp=X'*Y;% avoiding assessing that each time
XpX=X'*X;
iter=0;
krange=pcatype.eigenval.d(range);
for k=krange
   iter=iter+1;
   %disp(iter);
   %disp(k);
   beta(:,iter)=inv((XpX+k*eye(p)))*tmp;
   %beta(:,iter)=pinv((XpX+k*eye(p)))*tmp;
   predy(:,iter)=X*beta(:,iter)+meanY;   
   tmp1=predy(:,iter)-y.d;
   rmsec(iter)=sqrt(sum(tmp1.*tmp1)/n);
	%disp(rmsec(iter));   
end
% beta from original data
ridgetype.beta.d=pcatype.eigenvec.d*beta/sqrt(n);
ridgetype.beta.i=pcatype.eigenvec.i;
ridgetype.beta.v=num2str(krange');
ridgetype.krange.v=num2str(range');
ridgetype.krange.d=krange;
ridgetype.krange.i='k /component';
ridgetype.averagex.d=meanX;
ridgetype.averagex.i='Average of X';
ridgetype.averagex.v=pcatype.average.v;

ridgetype.averagey.d=meanY;
ridgetype.averagey.i='Average of Y';
ridgetype.averagey.v=y.v;

ridgetype.rmsec.d=rmsec;
ridgetype.rmsec.v=num2str(krange');
ridgetype.rmsec.i='root mean square sum of square';

ridgetype.predy.d=predy;
ridgetype.predy.v=num2str(krange');
ridgetype.predy.i=pcatype.score.i;

corr.d=zeros(1,iter);
for i=1:iter
   aux=corrcoef(y.d,predy(:,i));
   ridgetype.corr.d(1,i)=aux(1,2);
end;   
ridgetype.corr.i='correlation coefficient';
ridgetype.corr.v=num2str(krange');



